#pragma once

#include "iprinter.h"
#include "typeDefinitions.h"
#include <chrono>

class Cam : public IPrinter
{
	Q_OBJECT

public:
	enum State {
		disconnected,
		closed,
		opened,
		recording
	};

	Cam(QString name, QString ip);
	Cam(const Cam&) = delete; 		// Delete copy constructor and assignment operator
	Cam& operator=(const Cam&) = delete;
	~Cam();

	QString id();
	QString name() const;
	QString ip() const;
	RecordPtr rec();


	State state() const;
	void setState(const State&);

	QString stateToString(const State&);

	void onChanged(UpdateTriggerType);

	void open();
	void close();

	QString camInfo();
	QStringList getCamFeatures( QStringList );
	QStringList getStreamFeatures( QStringList );


	void saveSettings();
	void loadSettings();

	inline CameraPtr getCameraPtr() { return _vcam; }
	inline void setCameraPtr( CameraPtr pCamera ) { _vcam = pCamera; }

	friend class FrameObserver;
	friend class Core;

private:
	QStringList getFeatures(VmbCPP::FeatureContainerPtr, QStringList);

	CameraPtr _vcam;
	QString _name;
	QString _ip;
	QString _id;
	State _state;
	RecordPtr _rec;
};

