#pragma once

#include "../typeDefinitions.h"
#include <QThread>

// derived from qthread example
// https://doc.qt.io/qt-6/qthread.html

class Console : public QObject
{
	Q_OBJECT

public:
	static Console* getInstance();

	Console(const Console&) = delete; // Delete copy constructor and assignment operator to prevent copying
	Console& operator=(const Console&) = delete;

private:
	Console();

public slots:
	void listenKeys();

	void error(const QString&,const int&);
	void error(const int&);
	void warning(const QString&);
	void print(const QString&);
	void print(const QStringList&);

	void printVersion();
	void printHelp();
	// ...

signals:
	void keyPressed(const QChar &key);
	void lineEntered(const QString&);
	void numberEntered(const int&);

};

class Controller : public QObject
{
	Q_OBJECT
	QThread thread;

	public:
		Controller();
		~Controller();

	public slots:
		void onKeyPressed(const QChar &);
		void onLineEntered(const QString& line);
		void onNumberEntered(int number);

	signals:
		// to UI
		void operate();

		// to core
		void listCams();
		void detectCams();
		void openCam();
		void closeCam();
		void cam(int);
		void startRecording();
		void stopRecording();
		void showRecordingStats();
		void showInfo();
		void setDuration1(seconds);
		void setDurationAll(seconds);
		void loadSettings();
		void saveSettings();

		// void calibrateCam();
		// ... more signals here, new func XXX

	private:
		bool getDur1;
		bool getDurAll;

};
