#include "VmbCPP/Interface.h"

#include "console.h"
#include "../core.h"
#include "../utils.h"
#include "../cam.h"

#include "qdebug.h"
#include "qobjectdefs.h"
#include <QCoreApplication>
#include <QTimer>

#include <opencv2/core.hpp>

int main(int argc, char *argv[])
{

	QCoreApplication a(argc, argv);
	QCoreApplication::setApplicationVersion(utils::getVersion());
	QCoreApplication::setApplicationName("recorder");

	cv::setNumThreads(1); // since im only using cv in frameprocessor which is already multi-threaded.
	//xxx test performance


	//parse args
	QStringList arguments = a.arguments();
	for (int i = 1; i < arguments.size(); ++i)
	{
		if (arguments.at(i).endsWith(".json"))
			utils::configFile(arguments.at(i));
		else if (arguments.at(i).endsWith(".xml"))
			utils::settingsFile(arguments.at(i));
	}

	//init
	Core core;
	Controller controller;

	//connect ui (controller) -> vmb (core)
	QObject::connect(&controller, &Controller::listCams,		&core, &Core::listCams);
	QObject::connect(&controller, &Controller::detectCams,		&core, &Core::detectCams);
	QObject::connect(&controller, &Controller::openCam, 		&core, &Core::openCam);
	QObject::connect(&controller, &Controller::closeCam, 		&core, &Core::closeCam);
	QObject::connect(&controller, &Controller::cam, 			&core, &Core::cam);
	QObject::connect(&controller, &Controller::startRecording, 	&core, &Core::startRecording);
	QObject::connect(&controller, &Controller::stopRecording,	&core, &Core::stopRecording);
	QObject::connect(&controller, &Controller::showRecordingStats,	&core, &Core::showRecordingStats);
	QObject::connect(&controller, &Controller::showInfo, 		&core, &Core::showInfo);
	QObject::connect(&controller, &Controller::setDuration1,	&core, &Core::setDuration1);
	QObject::connect(&controller, &Controller::setDurationAll, 	&core, &Core::setDurationAll);
	QObject::connect(&controller, &Controller::loadSettings, 	&core, &Core::loadSettings);
	QObject::connect(&controller, &Controller::saveSettings, 	&core, &Core::saveSettings);
	// XXX add new func

	QTimer::singleShot(0, Console::getInstance(), SLOT(printVersion()));
	QTimer::singleShot(4000, &core, SLOT(openCam())); // for testing

	return a.exec();
}
