#include "frameobserver.h"
#include "frameprocessor.h"
#include "cam.h"
#include "record.h"

#include <VmbCPP/SharedPointerDefines.h>

#include <QThreadPool>
#include <QThread>

using namespace VmbCPP;

FrameObserver::FrameObserver( CamPtr cam ) : IFrameObserver( cam->getCameraPtr() ),
	_cam(cam),
	_processor(new FrameProcessor(_cam)),
	_pool(QThreadPool::globalInstance())
{
	// _pool->setMaxThreadCount(QThread::idealThreadCount()); //dflt
	connect(_processor, &FrameProcessor::frameProcessed, this, &FrameObserver::queueFrame, Qt::QueuedConnection);
}


FrameObserver::~FrameObserver()
{
	// qDebug() << __LINE__ << "-" << __PRETTY_FUNCTION__ << "";
	delete _processor;
}

/* Frame callback notifies about incoming frames
	process data afap
	some functions are prohibited within this callback!
	Do not apply image processing within this callback ( performance )
	-> create frame processor to handle frame processing asynchronously
	- when the frame has been processed , requeue it
	- frameprocessing is done in a separate thread
*/
void FrameObserver::FrameReceived ( const FramePtr pframe )
{
	// qDebug() << __LINE__ << "-" << __PRETTY_FUNCTION__ << "";

	VmbFrameStatusType status;
	pframe->GetReceiveStatus(status);
	if( status != VmbFrameStatusComplete )
	{
		queueFrame(pframe);
		return;
	}

	_processor->setFrame(pframe);
	_pool->start(_processor); //start processing in a separate thread. sa trystart()
}

void FrameObserver::queueFrame(FramePtr pframe)
{
	// qDebug() << __LINE__ << "-" << __PRETTY_FUNCTION__ << "";
	_cam->getCameraPtr()->QueueFrame(pframe); // Queue the frame back to the camera
}
